@@ -1,18 +1,18 @@
-#  Drakkar-Software OctoBot
-#  Copyright (c) Drakkar-Software, All rights reserved.
+#  This file is part of OctoBot (https://github.com/Drakkar-Software/OctoBot)
+#  Copyright (c) 2021 Drakkar-Software, All rights reserved.
 #
-#  This library is free software; you can redistribute it and/or
-#  modify it under the terms of the GNU Lesser General Public
-#  License as published by the Free Software Foundation; either
+#  OctoBot is free software; you can redistribute it and/or
+#  modify it under the terms of the GNU General Public License
+#  as published by the Free Software Foundation; either
 #  version 3.0 of the License, or (at your option) any later version.
 #
-#  This library is distributed in the hope that it will be useful,
+#  OctoBot is distributed in the hope that it will be useful,
 #  but WITHOUT ANY WARRANTY; without even the implied warranty of
 #  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-#  Lesser General Public License for more details.
+#  General Public License for more details.
 #
-#  You should have received a copy of the GNU Lesser General Public
-#  License along with this library.
+#  You should have received a copy of the GNU General Public
+#  License along with OctoBot. If not, see <https://www.gnu.org/licenses/>.
 import octobot.backtesting as backtesting
 import octobot_backtesting.constants as constants
 
@@ -24,13 +24,17 @@ def create_independent_backtesting(config,
                                    join_backtesting_timeout=constants.BACKTESTING_DEFAULT_JOIN_TIMEOUT,
                                    run_on_common_part_only=True,
                                    start_timestamp=None,
-                                   end_timestamp=None) -> backtesting.IndependentBacktesting:
+                                   end_timestamp=None,
+                                   enable_logs=True,
+                                   stop_when_finished=False) -> backtesting.IndependentBacktesting:
     return backtesting.IndependentBacktesting(config, tentacles_setup_config, data_files,
                                               data_file_path,
                                               run_on_common_part_only=run_on_common_part_only,
                                               join_backtesting_timeout=join_backtesting_timeout,
                                               start_timestamp=start_timestamp,
-                                              end_timestamp=end_timestamp)
+                                              end_timestamp=end_timestamp,
+                                              enable_logs=enable_logs,
+                                              stop_when_finished=stop_when_finished)
 
 
 async def initialize_and_run_independent_backtesting(independent_backtesting, log_errors=True) -> None:
@@ -49,6 +53,10 @@ async def stop_independent_backtesting(independent_backtesting, memory_check=Fal
     await independent_backtesting.stop(memory_check=memory_check, should_raise=should_raise)
 
 
+async def join_independent_backtesting_stop(independent_backtesting, timeout=None):
+    await independent_backtesting.join_stop_event(timeout=timeout)
+
+
 def check_independent_backtesting_remaining_objects(independent_backtesting) -> None:
     independent_backtesting.octobot_backtesting.check_remaining_objects()
 
@@ -66,7 +74,7 @@ def get_independent_backtesting_progress(independent_backtesting) -> float:
 
 
 def is_independent_backtesting_finished(independent_backtesting) -> bool:
-    return independent_backtesting.get_progress() == 1.0
+    return independent_backtesting.has_finished()
 
 
 def is_independent_backtesting_stopped(independent_backtesting) -> bool:
@@ -81,5 +89,9 @@ def get_independent_backtesting_exchange_manager_ids(independent_backtesting) ->
     return independent_backtesting.octobot_backtesting.exchange_manager_ids
 
 
+def get_independent_backtesting_bot_id(independent_backtesting) -> str:
+    return independent_backtesting.octobot_backtesting.bot_id
+
+
 def log_independent_backtesting_report(independent_backtesting) -> None:
     independent_backtesting.log_report()
