@@ -1,18 +1,18 @@
-#  Drakkar-Software OctoBot
-#  Copyright (c) Drakkar-Software, All rights reserved.
+#  This file is part of OctoBot (https://github.com/Drakkar-Software/OctoBot)
+#  Copyright (c) 2021 Drakkar-Software, All rights reserved.
 #
-#  This library is free software; you can redistribute it and/or
-#  modify it under the terms of the GNU Lesser General Public
-#  License as published by the Free Software Foundation; either
+#  OctoBot is free software; you can redistribute it and/or
+#  modify it under the terms of the GNU General Public License
+#  as published by the Free Software Foundation; either
 #  version 3.0 of the License, or (at your option) any later version.
 #
-#  This library is distributed in the hope that it will be useful,
+#  OctoBot is distributed in the hope that it will be useful,
 #  but WITHOUT ANY WARRANTY; without even the implied warranty of
 #  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-#  Lesser General Public License for more details.
+#  General Public License for more details.
 #
-#  You should have received a copy of the GNU Lesser General Public
-#  License along with this library.
+#  You should have received a copy of the GNU General Public
+#  License along with OctoBot. If not, see <https://www.gnu.org/licenses/>.
 import time
 import asyncio
 import json
@@ -23,6 +23,7 @@ import octobot_commons.logging as logging
 import octobot_commons.configuration as configuration
 import octobot_commons.os_util as os_util
 import octobot_commons.symbol_util as symbol_util
+import octobot_commons.authentication as authentication
 
 import octobot_commons.constants as common_constants
 
@@ -193,7 +194,7 @@ class CommunityManager:
                and trading_api.is_sponsoring(exchange_name) \
                and trading_api.is_valid_account(exchange_manager):
                 supporting_exchanges.append(exchange_name)
-        supports = self.octobot_api.get_community_auth().supports
+        supports = authentication.Authenticator.instance().supports
         return {
             community_fields.CommunityFields.EXCHANGES.value: supporting_exchanges,
             community_fields.CommunityFields.ROLES.value: [supports.support_role],
